@@ -41,9 +41,7 @@ def test_openapi_schema(client: TestClient):
                             "description": "The created item",
                             "content": {
                                 "application/json": {
-                                    "schema": {
-                                        "$ref": "#/components/schemas/ItemOutput"
-                                    }
+                                    "schema": {"$ref": "#/components/schemas/Item"}
                                 }
                             },
                         },
@@ -64,7 +62,7 @@ def test_openapi_schema(client: TestClient):
                     "requestBody": {
                         "content": {
                             "application/json": {
-                                "schema": {"$ref": "#/components/schemas/ItemInput"}
+                                "schema": {"$ref": "#/components/schemas/Item"}
                             }
                         },
                         "required": True,
@@ -74,7 +72,7 @@ def test_openapi_schema(client: TestClient):
         },
         "components": {
             "schemas": {
-                "ItemInput": {
+                "Item": {
                     "title": "Item",
                     "required": ["name", "price"],
                     "type": "object",
@@ -98,30 +96,6 @@ def test_openapi_schema(client: TestClient):
                         },
                     },
                 },
-                "ItemOutput": {
-                    "title": "Item",
-                    "required": ["name", "description", "price", "tax", "tags"],
-                    "type": "object",
-                    "properties": {
-                        "name": {"title": "Name", "type": "string"},
-                        "description": {
-                            "anyOf": [{"type": "string"}, {"type": "null"}],
-                            "title": "Description",
-                        },
-                        "price": {"title": "Price", "type": "number"},
-                        "tax": {
-                            "title": "Tax",
-                            "anyOf": [{"type": "number"}, {"type": "null"}],
-                        },
-                        "tags": {
-                            "title": "Tags",
-                            "uniqueItems": True,
-                            "type": "array",
-                            "items": {"type": "string"},
-                            "default": [],
-                        },
-                    },
-                },
                 "ValidationError": {
                     "title": "ValidationError",
                     "required": ["loc", "msg", "type"],
